import { useMemo } from 'react';
import { RGBDifference } from '../../color_functions/color_module';
import { useColorFunction } from '../../ColorFunctionContext';
import styles from './CompareLayout.module.css';
import { CompareMethodProps } from './share-props';

const defaultCompareResult: RGBDifference = {
  r: {
    delta: 0,
    percent: 0,
  },
  g: {
    delta: 0,
    percent: 0,
  },
  b: {
    delta: 0,
    percent: 0,
  },
};

export function RGBCompare({
  basic = '000000',
  compare = '000000',
  mode = 'absolute',
}: CompareMethodProps) {
  const { colorFn } = useColorFunction();
  const differ = useMemo(() => {
    if (!colorFn) {
      return defaultCompareResult;
    }
    try {
      const diffFn = {
        absolute: colorFn.differ_in_rgb,
        relative: colorFn.relative_differ_in_rgb,
      };
      const diff = diffFn[mode](basic, compare);
      return {
        r: {
          delta: Math.round(diff.r.delta * 255),
          percent: diff.r.percent,
        },
        g: {
          delta: Math.round(diff.g.delta * 255),
          percent: diff.g.percent,
        },
        b: {
          delta: Math.round(diff.b.delta * 255),
          percent: diff.b.percent,
        },
      } as RGBDifference;
    } catch (e) {
      console.error('[Compare in RGB]', e);
    }
    return defaultCompareResult;
  }, [basic, compare, mode]);

  return (
    <div>
      <h6>Compare in RGB Space</h6>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>R</div>
          <div className={styles.element_values}>
            <span>{differ.r.delta}</span>
            <span>{isNaN(differ.r.percent) ? '0.00' : (differ.r.percent * 100).toFixed(2)}%</span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>G</div>
          <div className={styles.element_values}>
            <span>{differ.g.delta}</span>
            <span>{isNaN(differ.g.percent) ? '0.00' : (differ.g.percent * 100).toFixed(2)}%</span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>B</div>
          <div className={styles.element_values}>
            <span>{differ.b.delta}</span>
            <span>{isNaN(differ.b.percent) ? '0.00' : (differ.b.percent * 100).toFixed(2)}%</span>
          </div>
        </div>
      </div>
    </div>
  );
}
